------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: aenea-gui-handlers.adb,v 1.9 2004/03/10 23:49:58 Jano Exp $

with Aenea.Countries;
with Aenea.Globals;
with Aenea.Globals.Options;
with Aenea.Gui.Events;
with Aenea.Hub.Topo;
with Aenea.Misc;
with Aenea.Net;
with Aenea.Walker;
with Event_queue;
with Strings.Utils;

with Agpl.Bmp;
with Agpl.Http.Server;
with Agpl.Http.Server.Simple_handler;
with Agpl.Http.Server.Sort_handler;

with Aws.Messages;
with Aws.Mime;
with Aws.Parameters;
with Aws.Response;
with Aws.Status;
with Templates_parser;
use  Templates_parser;

with Ada.Calendar;
use  Ada;

package body Aenea.Gui.Handlers is

   use type Calendar.Time;

   -- Startup:
   Startup : Calendar.Time := Calendar.Clock;

   ------------------------------------------------------------------------
   -- Raw_handler                                                        --
   ------------------------------------------------------------------------
   function Raw_handler (Request : in Aws.Status.Data) 
      return Aws.Response.Data is
      URI : constant String := Agpl.Http.Server.Get_root & "raw.html";
   begin
      declare
         Translat : Translate_table := (
            Assoc ("VERSION", Version),
            Assoc ("hubs", Net.Counter.Hubs_count),
            Assoc ("leaves", Net.Counter.Leaves_count),
            Assoc ("leavesperhub", 
               Misc.To_string (
                  Float (Net.Counter.Leaves_count) /
                  Float (Integer'Max (Net.Counter.Hubs_count, 1)))),
            Assoc ("tracked", Walker.Adder.Count),
            Assoc ("events", Event_queue.Length (Net.Events)),
            Assoc ("packets", Net.Outgoing.Length),
            Assoc ("uptime", 
               Strings.Utils.Trim (Misc.Image (Calendar.Clock - Startup)))
            );
      begin
         return Aws.Response.Build (
            Aws.Mime.Content_type (URI),
            String'(Parse (URI, Translat & Agpl.Http.Server.Standard_xlats (Request))),
            Cache_control => Aws.Messages.No_cache);
      end;
   end Raw_handler;

   ------------------------------------------------------------------------
   -- Topogram_handler                                                   --
   ------------------------------------------------------------------------
   function Topogram_handler (Request : in Aws.Status.Data)
      return Aws.Response.Data
   is
      Params : Aws.Parameters.List := Aws.Status.Parameters (Request);
      W, H   : Natural;
      Kind   : Hub.Topo.Kinds;
   begin
      begin
         Kind := Hub.Topo.Kinds'Value (Aws.Parameters.Get (Params, "kind"));
      exception
         when others =>
            Kind := Hub.Topo.Kinds'Value (S (Globals.Options.Topo_type));
      end;
      begin
         W := Natural'Value (Aws.Parameters.Get (Params, "width"));
      exception
         when others =>
            W := Globals.Options.Topo_width;
      end;
      begin
         H := Natural'Value (Aws.Parameters.Get (Params, "height"));
      exception
         when others =>
            H := Globals.Options.Topo_height;
      end;
      return Aws.Response.Build (
         Agpl.Bmp.Mime_type,
         Bmp.Get_stream (
            Walker.Topogram (Width => W, Height => H, Kind => Kind)),
         Cache_control => Aws.Messages.No_cache);
   end Topogram_handler;

   function Command_handler (Request : in Aws.Status.Data)
      return Aws.Response.Data
   is
      Params : Aws.Parameters.List := Aws.Status.Parameters (Request);
      Action : constant String := Aws.Parameters.Get (Params, "action");
   begin
      if Action = "reload_config" then
         Globals.Init;
         Globals.Options.Set_options;
      else
         raise Unimplemented;
      end if;
      return Aws.Response.Build (
         Aws.Mime.Text_plain, "Command applied");
   end Command_handler;

   Raw_object : Agpl.Http.Server.Simple_handler.Object (Raw_handler'Access);
   Hub_object : Agpl.Http.Server.Sort_handler.Object (
      Source => Walker.Report'Access, 
      Single => Agpl.Http.Server.Sort_handler.Void_singleton'Access,
      Page   => new String'("hubs.html"));
   Countries_object : Agpl.Http.Server.Sort_handler.Object (
      Source => Countries.Report'Access, 
      Single => Net.Report_hubs'Access,
      Page   => new String'("countries.html"));
   Events_object : Agpl.Http.Server.Sort_handler.Object (
      Source => Gui.Events.Records.Http_Report'Access, 
      Single => Agpl.Http.Server.Sort_handler.Null_singleton'Access,
      Page   => new String'("events.html"));
   Topogram_object : 
      Agpl.Http.Server.Simple_handler.Object (Topogram_handler'Access);
   -- De-linked handlers:
   Hub2_object : Agpl.Http.Server.Sort_handler.Object (
      Source => Walker.Report'Access, 
      Single => Agpl.Http.Server.Sort_handler.Void_singleton'Access,
      Page   => new String'("hubs2.html"));
   Countries2_object : Agpl.Http.Server.Sort_handler.Object (
      Source => Countries.Report'Access, 
      Single => Net.Report_hubs'Access,
      Page   => new String'("countries2.html"));
   Command_object : 
      Agpl.Http.Server.Simple_handler.Object (Command_handler'Access);

begin
   -- Skin:
   Agpl.Http.Server.Set_root (S (Globals.Options.Gui_HtmlRoot));
   Agpl.Http.Server.Set_style_sheet (S (Globals.Options.Gui_HtmlStyle));

   -- Registrations:
   Agpl.Http.Server.Register_handler ("/raw.html",    Raw_object);
   Agpl.Http.Server.Register_handler ("/hubs.html",   Hub_object);
   Agpl.Http.Server.Register_handler ("/countries.html", Countries_object);
   Agpl.Http.Server.Register_handler ("/events.html", Events_object);
   Agpl.Http.Server.Register_handler ("/topogram",    Topogram_object);
   Agpl.Http.Server.Register_handler ("/hubs2.html",  Hub2_object);
   Agpl.Http.Server.Register_handler ("/countries2.html", Countries2_object);
   Agpl.Http.Server.Register_handler ("/command",     Command_object);

   Agpl.Http.Server.Set_server_name (Version);
end Aenea.Gui.Handlers;
