------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: aenea-hub.adb,v 1.10 2004/03/22 07:14:55 Jano Exp $

with Aenea.Gui.Events;
with Aenea.Net;
with Aenea.Trace;

with Adagio.G2.Packet;
with Adagio.G2.Packet.Queue;
with Adagio.Socket;

package body Aenea.Hub is

   ------------------------------------------------------------------------
   -- Create                                                             --
   ------------------------------------------------------------------------
   procedure Create (
      This    : out Object; 
      Address : in String; 
      Leaves  : in Natural := 0;
      Alive   : in Boolean := false) 
   is
   begin
      Gui.Events.Add ("[+] Create - " & Address, 2);
      This.Address    := U (Address);
      This.Leaves     := Leaves;
      This.Max_leaves := 0;
      This.Pings      := 0;
      This.Alive      := Alive;
      This.Failures   := 0;
      -- Clear (This.Neighbors);
      This.Stat_First_seen := Calendar.Clock;
      This.Stat_Last_seen  := Calendar.Clock;

      -- Location:
      This.Country_code := Agpl.Geoip.Country_code_from_addr (Address);
      This.Country_name := U (Agpl.Geoip.Country_name_from_addr (Address));
   end Create;

   ------------------------------------------------------------------------
   -- Id                                                                 --
   ------------------------------------------------------------------------
   -- Returns a string uniquely identifying the hub (ip:port)
   function Id (This : in Object) return String is
   begin
      return S (This.Address);
   end Id;
   
   ------------------------------------------------------------------------
   -- Address                                                            --
   ------------------------------------------------------------------------
   -- Return a socket address
   function Address (This : in Object) return
      Adagio.Socket.Sock_addr_type
   is
   begin
      return Adagio.Socket.To_address (Id (This));
   end Address;

   ------------------------------------------------------------------------
   -- Stats                                                              --
   ------------------------------------------------------------------------
   function Stats (This : in Object) return String is
   begin
      return 
         This.Stat_queries'Img & 
         This.Stat_pings'Img & 
         This.Stat_answers'Img & " " &
         This.Alive'Img;
   end Stats;

   ------------------------------------------------------------------------
   -- Kill                                                               --
   ------------------------------------------------------------------------
   procedure Kill (This : in out Object) is
   begin
      if This.Alive and This.Leaves > 0 then
         Gui.Events.Add ("[-] Kill   - " & Id (This), 3);
      elsif This.Alive then
         Gui.Events.Add ("[-] Kill   - " & Id (This), 2);
      else
         Gui.Events.Add ("[-] Kill   - " & Id (This), 1);
      end if;
      This.Alive            := false;
      This.Answer_pending   := false;
      This.Stat_Answers_in_a_row := 0;
      if This.Leaves > 0 then
         Net.Counter.Add (Leaves => -This.Leaves);
         This.Leaves         := 0;
      end if;
   end Kill;

   ------------------------------------------------------------------------
   -- Ping                                                               --
   ------------------------------------------------------------------------
   -- Launchs a ping
   procedure Ping (This : in out Object) is
      P : G2.Packet.Object := G2.Packet.Create ("PI");
   begin
      Gui.Events.Add ("[>] PI     - " & Id (This), 1);
      Trace.Log ("PINGING " & Id (This) & Stats (This), Trace.Never);
      This.Stat_pings     := This.Stat_pings   + 1;
      This.Answer_pending := true;
      This.Pings          := This.Pings + 1;

      G2.Packet.Queue.Send (
         Net.Outgoing, 
         P,
         Address (This),
         Safe => true);
   end Ping;

   ------------------------------------------------------------------------
   -- Query                                                              --
   ------------------------------------------------------------------------
   -- Launchs a query. If another was pending, returns without sending.
   procedure Query (This : in out Object) is
      P : G2.Packet.Object := G2.Packet.Create ("CRAWLR");
   begin
      Gui.Events.Add ("[>] CRAWLR - " & Id (This), 1);
      Trace.Log ("QUERYING " & Id (This) & Stats (This), Trace.Never);
      This.Stat_queries   := This.Stat_queries + 1;
      This.Answer_pending := true;

      G2.Packet.Queue.Send (
         Net.Outgoing, 
         P,
         Address (This),
         Safe => true);
   end Query;

   ------------------------------------------------------------------------
   -- Set_alive                                                          --
   ------------------------------------------------------------------------
   procedure Set_alive (This : in out Object; Alive : in Boolean := true) is
   begin
      if Alive and then not This.Alive then
         This.Stat_First_seen := Calendar.Clock;
         This.Stat_Last_seen  := Calendar.Clock;
         Gui.Events.Add ("[+] Alive  - " & Id (This), 3);
      end if;
      This.Alive := Alive;
   end Set_alive;

   ------------------------------------------------------------------------
   -- Set_leaves                                                         --
   ------------------------------------------------------------------------
   procedure Set_leaves (This : in out Object; Leaves : Natural) is
   begin
      This.Leaves := Leaves;
   end Set_leaves;

   ------------------------------------------------------------------------
   -- Leaves                                                             --
   ------------------------------------------------------------------------
   function Leaves (This : in Object) return Natural is
   begin
      return This.Leaves;
   end Leaves;

   ------------------------------------------------------------------------
   -- Alive                                                              --
   ------------------------------------------------------------------------
   function Alive (This : in Object) return Boolean is
   begin
      return This.Alive;
   end Alive;

   ------------------------------------------------------------------------
   -- Set_answer_pending                                                 --
   ------------------------------------------------------------------------
   procedure Set_answer_pending (
      This : in out Object; Answer_pending : in Boolean := true) is
   begin
      This.Answer_pending := Answer_pending;
   end Set_answer_pending;

   ------------------------------------------------------------------------
   -- Answer_pending                                                     --
   ------------------------------------------------------------------------
   function Answer_pending (This : in Object) return Boolean is
   begin
      return This.Answer_pending;
   end Answer_pending;

   ------------------------------------------------------------------------
   -- Set_max_leaves                                                     --
   ------------------------------------------------------------------------
   procedure Set_max_leaves (This : in out Object; Leaves : Natural) is
   begin
      This.Max_leaves := Leaves;
   end Set_max_leaves;

   ------------------------------------------------------------------------
   -- Max_leaves                                                         --
   ------------------------------------------------------------------------
   function Max_leaves (This : in Object) return Natural is
   begin
      return This.Max_leaves;
   end Max_leaves;

   ------------------------------------------------------------------------
   -- Add_answer                                                         --
   ------------------------------------------------------------------------
   -- Adds 1 to the answers statistic
   procedure Add_answer (This : in out Object) is
   begin
      This.Stat_answers          := This.Stat_answers + 1;
      This.Stat_answers_in_a_row := This.Stat_answers_in_a_row + 1;
      This.Stat_Last_seen        := Calendar.Clock;
   end Add_answer;

   ------------------------------------------------------------------------
   -- Set_scheduled                                                      --
   ------------------------------------------------------------------------
   procedure Set_scheduled (This : in out Object; Sched : in Boolean := true)
   is
   begin
      This.Scheduled := Sched;
   end Set_scheduled;

   ------------------------------------------------------------------------
   -- Scheduled                                                          --
   ------------------------------------------------------------------------
   function Scheduled (This : in Object) return Boolean is
   begin 
      return This.Scheduled;
   end Scheduled;

   ------------------------------------------------------------------------
   -- Failures                                                           --
   ------------------------------------------------------------------------
   function Failures (This : in Object) return Natural is
   begin
      return This.Failures;
   end Failures;

   ------------------------------------------------------------------------
   -- Set_failures                                                       --
   ------------------------------------------------------------------------
   procedure Set_failures (This : in out Object; Failures : in Natural := 0)
   is
   begin
      This.Failures := Failures;
   end Set_failures;

   ------------------------------------------------------------------------
   -- Relocate                                                           --
   ------------------------------------------------------------------------
   -- Re-checks the location, just in case the IP range has been moved.
   procedure Relocate (This : in out Object) is
   begin
      This.Country_code := 
         Agpl.Geoip.Country_code_from_addr (Id (This));
      This.Country_name := 
         U (Agpl.Geoip.Country_name_from_addr (Id (This)));
   end Relocate;

   ------------------------------------------------------------------------
   -- Get_country_code                                                   --
   ------------------------------------------------------------------------
   function Get_country_code (This : in Object) 
      return Agpl.Geoip.Country_code is
   begin
      return This.Country_code;
   end Get_country_code;

   ------------------------------------------------------------------------
   -- Clear neighbors                                                    --
   ------------------------------------------------------------------------
   procedure Clear_neighbors (This : in out Object) is
   begin
      Links_map.Clear (This.Neighbors);
   end Clear_neighbors;

   ------------------------------------------------------------------------
   -- Add_neighbor                                                       --
   ------------------------------------------------------------------------
   procedure Add_neighbor (This : in out Object; Neigh : in Object_access) is
      -- Search already there:
      I : Links_map.Iterator_type;
   begin
      if Id (This) /= Id (Neigh.all) then
         I := Links_map.Find (This.Neighbors, Id (Neigh.all));
         if I /= Links_map.Back (This.Neighbors) then
            Links_map.Insert (This.Neighbors, Id (Neigh.all), 
               (Neigh, Links_map.Element (I).Since));
         else
            Links_map.Insert (This.Neighbors, Id (Neigh.all), 
               (Neigh, Calendar.Clock));
         end if;
      end if;
   end Add_neighbor;

   ------------------------------------------------------------------------
   -- Make_neighbors                                                     --
   ------------------------------------------------------------------------
   -- Marks both hubs as neighbors, preserving the oldest reach time.
   procedure Make_neighbors (L, R : access Object) is
      Since : Calendar.Time := Calendar.Clock;
      I     : Links_map.Iterator_type;
   begin
      if Id (L.all) /= Id (R.all) then
         -- Search in source:
         I := Links_map.Find (L.Neighbors, Id (R.all));
         if I /= Links_map.Back (L.Neighbors) then
            Since := Links_map.Element (I).Since;
         end if;
         -- Search in destination:
         I := Links_map.Find (R.Neighbors, Id (L.all));
         if I /= Links_map.Back (L.Neighbors) then
            if Links_map.Element (I).Since < Since then
               Since := Links_map.Element (I).Since;
            end if;
         end if;
         -- Insert in both hubs:
         Links_map.Insert (L.Neighbors, Id (R.all), 
            (Object_access (R), Since));
         Links_map.Insert (R.Neighbors, Id (L.all), 
            (Object_access (L), Since));
      end if;
   end Make_neighbors;

   ------------------------------------------------------------------------
   -- Get_neighbors                                                      --
   ------------------------------------------------------------------------
   procedure Get_neighbors (
      This : in Object; Neighbors : out Links_map.Container_type)
   is
      procedure Insert (This : in Links_map.Iterator_type) is
         use Links_map;
      begin
         Insert (Neighbors, Key (This), Element (This));
      end Insert;
      procedure Copy is new Links_map.Generic_iteration (Insert);
   begin
      Links_map.Clear (Neighbors);
      Copy (This.Neighbors, 
         Links_map.First (This.Neighbors), Links_map.Back (This.Neighbors));
   end Get_neighbors;

   ------------------------------------------------------------------------
   -- Same_link                                                          --
   ------------------------------------------------------------------------
   -- Only takes into account the id of the destination peer:
   function Same_link (L, R : in Links) return Boolean is
   begin
      return Id (L.Peer.all) = Id (R.Peer.all);
   end Same_link;

   ------------------------------------------------------------------------
   -- Alive_neighbors                                                    --
   ------------------------------------------------------------------------
   function Alive_neighbors (This : in Object) return Natural is
      I   : Links_map.Iterator_type := Links_map.First (This.Neighbors);
      Num : Natural := 0;
   begin
      while I /= Links_map.Back (This.Neighbors) loop
         if Alive (Links_map.Element (I).Peer.all) and then
            Leaves (Links_map.Element (I).Peer.all) > 0
         then
            Num := Num + 1;
         end if;
         I := Links_map.Succ (This.Neighbors, I);
      end loop;
      return Num;
   end Alive_neighbors;

   ------------------------------------------------------------------------
   -- Get_link_duration                                                  --
   ------------------------------------------------------------------------
   -- Says for how many time a hub has been connected to another
   function Get_link_duration (This : in Object; Neigh : in String) 
      return Duration
   is
      I : Links_map.Iterator_type := Links_map.Find (This.Neighbors, Neigh);
   begin
      return Calendar.Clock - Links_map.Element (I).Since;
   end;

end Aenea.Hub;
