------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: aenea-net.adb,v 1.5 2004/01/26 20:47:15 Jano Exp $

with Adagio.G2.Packet.Queue;
with Adagio.G2.Transceiver_Types;
with Adagio.Globals.Options;
with Adagio.Socket;

package body Aenea.Net is

   function "/" (L : Natural; R : Integer) return Float is
   begin
      return Float (L) / Float (R);
   end "/";

   -- UDP socket
   UDPS : aliased Adagio.Socket.Object;

   ------------------------------------------------------------------------
   -- Init                                                               --
   ------------------------------------------------------------------------
   -- Do all necessary net initialization
   procedure Init is
   begin
      -- Create listener socket
      Adagio.Socket.Create_datagram (UDPS);
      Adagio.Socket.Bind (UDPS, "0.0.0.0", Adagio.Globals.Options.G2_port);
      -- Transceiver
      UDPT := new Adagio.G2.Transceiver.Object;
      Adagio.G2.Transceiver.Start (UDPT.all, UDPS'Access, Incoming'Access);
      -- Queue limits
      G2.Packet.Queue.Max_pending_packets := Natural'Last;
      G2.Transceiver_Types.Max_packets    := Natural'Last;
   end Init;

   ------------------------------------------------------------------------
   -- Shutdown                                                           --
   ------------------------------------------------------------------------
   procedure Shutdown is
   begin
      Event_queue.Shutdown (Events);
      Adagio.G2.Transceiver.Shutdown (UDPT.all);
      Adagio.Socket.Close (UDPS);
   end Shutdown;

   ------------------------------------------------------------------------
   -- Counter                                                            --
   ------------------------------------------------------------------------
   -- Encapsulates host counts
   protected body Counter is

      ---------
      -- Add --
      ---------
      procedure Add (
         Hubs       : Integer := 0; 
         Leaves     : Integer := 0;
         Max_leaves : Integer := 0) is
      begin
         -- Trace.Log (Counter.Hubs'Img & Hubs'Img & Counter.Leaves'Img & Leaves'Img, Trace.Always);
         Counter.Hubs       := Counter.Hubs   + Hubs;
         Counter.Leaves     := Counter.Leaves + Leaves;
         Counter.Max_leaves := Counter.Max_leaves + Max_leaves;
      end Add;

      ----------
      -- Hubs --
      ----------
      function Hubs_count return Natural is
      begin
         return Hubs;
      end Hubs_count;

      ------------
      -- Leaves --
      ------------
      function Leaves_count return Natural is
      begin
         return Leaves;
      end Leaves_count;

      ----------------
      -- Max leaves --
      ----------------
      function Max_leaves_count return Natural is
      begin
         return Max_leaves;
      end Max_leaves_count;

      ---------------
      -- Push_hubs --
      ---------------
      procedure Push_hubs (Hubs : Natural) is
      begin
         Avg.Push (Avg_hubsc, Hubs);
      end Push_hubs;
      function Avg_hubs return Natural is
      begin
         return Natural (Avg.Average (Avg_hubsc));
      end Avg_hubs;

      -----------------
      -- Push_leaves --
      -----------------
      procedure Push_leaves (Leaves : Natural) is
      begin
         Avg.Push (Avg_leavesc, Leaves);
      end Push_leaves;
      function Avg_leaves return Natural is
      begin
         return Natural (Avg.Average (Avg_leavesc));
      end Avg_leaves;

   end Counter;

   ------------------------------------------------------------------------
   -- Report_hubs                                                        --
   ------------------------------------------------------------------------
   -- Single translation
   -- SINGLE1 <-- Current hubs
   -- SINGLE2 <-- Averaged hubs
   function Report_hubs return Templates_parser.Translate_table is
      use Templates_parser;
      Avg : Natural;
   begin
      begin 
         Avg := Counter.Avg_hubs;
      exception
         when others =>
            Avg := Counter.Hubs_count;
      end;
      return (
         Assoc ("SINGLE1", Counter.Hubs_count),
         Assoc ("SINGLE2", Avg));
   end Report_hubs;

end Aenea.Net;
