------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: aenea-walker.ads,v 1.10 2004/03/09 23:46:49 Jano Exp $

with Aenea.Hub;
with Aenea.Hub.Topo;

with Agpl.Bmp;
with Agpl.Http.Server.Sort_handler;

with Adagio.G2.Packet.Queue;

with Event_queue;

package Aenea.Walker is

   ------------------------------------------------------------------------
   -- Start                                                              --
   ------------------------------------------------------------------------
   -- Restores hubs
   -- Creates the first event to process received packets.
   procedure Start;

   ------------------------------------------------------------------------
   -- Add_hub                                                            --
   ------------------------------------------------------------------------
   -- Adds a hub to the mesh of known hubs.
   -- It will be immediately queried if Query is true.
   procedure Add_hub (This : in Hub.Object; Query : in Boolean := true);

   ------------------------------------------------------------------------
   -- Count_hub                                                          --
   ------------------------------------------------------------------------
   -- Will increase Counter.Hubs and do other registrations.
   procedure Count_hub (This : in Hub.Object);

   ------------------------------------------------------------------------
   -- Discount_hub                                                       --
   ------------------------------------------------------------------------
   -- Will decrease Counter.Hubs and do other de-registrations.
   procedure Discount_hub (This : in Hub.Object);

   ------------------------------------------------------------------------
   -- Event_save_hubs                                                    --
   ------------------------------------------------------------------------
   procedure Event_save_hubs (Context : in Event_queue.Context_access);

   ------------------------------------------------------------------------
   -- Adder                                                              --
   ------------------------------------------------------------------------
   protected Adder is
      procedure Add_hub (This : in Hub.Object; Query : in Boolean := true);
      function  Count return Natural;
      procedure Save_hubs;
      procedure Restore_hubs;
      procedure Report (Data : out Agpl.Http.Server.Sort_handler.Data_set);
      procedure Process_item (Item : in G2.Packet.Queue.Item_type);
      function  Topogram (
         Width, Height : in Positive; Kind : in Hub.Topo.Kinds) 
         return Agpl.Bmp.Object;
      procedure Check_hub (Context : in Event_queue.Context_access);
      procedure Check_timeout (Context : in Event_queue.Context_access);
   end Adder;

   procedure Process_item (Item : in G2.Packet.Queue.Item_type) renames
      Adder.Process_item;

   ------------------------------------------------------------------------
   -- Check_hub                                                          --
   ------------------------------------------------------------------------
   -- Sends a query or ping for a hub upon event arrival.
   procedure Check_hub (Context : in Event_queue.Context_access);

   ------------------------------------------------------------------------
   -- Check_timeout                                                      --
   ------------------------------------------------------------------------
   -- Checks timeouts for a hub.
   -- May kill the hub if rules say that.
   -- The hub will remaing scheduled, the schedule is removed in Check_hub
   procedure Check_timeout (Context : in Event_queue.Context_access);

   ------------
   -- Report --
   ------------
   procedure Report (Data : out Agpl.Http.Server.Sort_handler.Data_set);

   --------------
   -- Topogram --
   --------------
   function  Topogram (
      Width, Height : in Positive; Kind : in Hub.Topo.Kinds) 
         return Agpl.Bmp.Object renames Adder.Topogram;

end Aenea.Walker;
