------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: average_queue.ads,v 1.3 2004/01/21 21:05:43 Jano Exp $

with Ada.Unchecked_deallocation;

package body Agpl.Average_queue.Timed is

   ------------------------------------------------------------------------
   -- Push                                                               --
   ------------------------------------------------------------------------
   procedure Push (This : in out Object; Data : in Item) is
   begin
      This.Safe.Push (Data);
   end Push;

   ------------------------------------------------------------------------
   -- Average                                                            --
   ------------------------------------------------------------------------
   procedure Average (This : in out Object; Result : out Item) is
   begin
      This.Safe.Avg (Result);
   end Average;

   ------------------------------------------------------------------------
   -- Safe_object                                                        --
   ------------------------------------------------------------------------
   protected body Safe_object is
      ----------
      -- Push --
      ----------
      procedure Push (Value : in Item) is
         Now        : Calendar.Time := Calendar.Clock;
         Empty_gaps : Natural;
      begin
         if Now - Slot_start > Gap then
            -- Push acum
            Push (Data.all, Acum);

            -- Zeroes for elapsed empty gaps
            Empty_gaps := Natural ((Now - Slot_start - Gap) / Gap);
            if Empty_gaps >= Data.Size then
               for N in 1 .. Data.Size loop
                  Push (Data.all, 0.0);
               end loop;
            else
               for N in 1 .. Empty_gaps loop
                  Push (Data.all, 0.0);
               end loop;
            end if;

            -- New acum:
            Acum := Value;

            -- New slot_start, the pushed one plus empty ones:
            Slot_start := Slot_start + Gap * (Empty_gaps + 1);
         else
            Acum := Acum + Value;
         end if;
      end Push;

      ---------
      -- Avg --
      ---------
      procedure Avg  (Result : out Item) is
      begin
         Push (0.0); -- Update to current time
         if Is_empty (Data.all) then
            Result := 0.0;
         else
            Result := Average (Data.all) / Item (Gap);
         end if;
      end Avg;

      ------------
      -- Create --
      ------------
      procedure Create is 
      begin
         Data := new Average_queue.Object (Size => Slots);
      end Create;

      -------------
      -- Destroy --
      -------------
      procedure Destroy is
         procedure Free is new Unchecked_deallocation (
            Average_queue.Object, Average_queue.Object_access);
      begin
         Free (Data);
      end Destroy;

   end Safe_object;

   procedure Initialize (This : in out Destroyer_type) is
   begin
      This.Parent.Safe.Create;
   end Initialize;

   procedure Finalize   (This : in out Destroyer_type) is
   begin
      This.Parent.Safe.Destroy;
   end Finalize;

end Agpl.Average_queue.Timed;
