------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: agpl-constants.ads,v 1.1 2004/02/24 15:26:09 Jano Exp $

with Charles.Hash_String;
with Charles.Maps.Hashed.Strings.Unbounded;

with Ada.Finalization;
use  Ada;

package Agpl.Counter.Multi is

   ------------------------------------------------------------------------
   -- Object                                                             --
   ------------------------------------------------------------------------
   type Object is limited private;

   ------------------------------------------------------------------------
   -- Add                                                                --
   ------------------------------------------------------------------------
   -- Creates it if doesn't exists with value Increment
   procedure Add (This : in out Object; Key : in String; Increment : in Integer := 1);

   ------------------------------------------------------------------------
   -- Reset                                                              --
   ------------------------------------------------------------------------
   procedure Reset (This : in out Object; Key : in String; Val : in Integer := 0);

   ------------------------------------------------------------------------
   -- Val                                                                --
   ------------------------------------------------------------------------
   function  Val (This : in Object; Key : in String) return Integer;

   ------------------------------------------------------------------------
   -- Max_Key                                                            --
   ------------------------------------------------------------------------
   function  Max_Key (This : in Object) return String; -- Key with highest value

private

   package Counter_Map is new Charles.Maps.Hashed.Strings.Unbounded (
      Counter.Object_Access, 
      Charles.Hash_String,
      "=",
      "=");

   ------------------------------------------------------------------------
   -- Safe                                                               --
   ------------------------------------------------------------------------
   protected type Safe_Object (Parent : access Object) is
      -- Creates it if doesn't exists with value Increment
      procedure Add (Key : in String; Increment : in Integer := 1);
      procedure Reset (Key : in String; Val     : in Integer := 0);
      function  Val (Key : in String) return Integer;
      function  Max_Key return String; -- Key with highest value
   private
      Values     : aliased Counter_Map.Container_Type;
   end Safe_Object;

   type Object is new Finalization.Limited_Controlled with record
      Safe : Safe_Object (Object'Access);
   end record;

   procedure Finalize (This : in out Object);

end Agpl.Counter.Multi;
