------------------------------------------------------------------------------
--                         ADAGIO - ADALID - AENEA.                         --
--                                                                          --
--                            Copyright (C) 2003                            --
--                                 A. Mosteo.                               --
--                                                                          --
--  Authors: A. Mosteo. (adagio@mosteo.com)                                 --
--                                                                          --
--  If you have any questions in regard to this software, please address    --
--  them to the above email.                                                --
--                                                                          --
--  This program is free software; you can redistribute it and/or modify    --
--  it under the terms of the GNU General Public License as published by    --
--  the Free Software Foundation; either version 2 of the License, or (at   --
--  your option) any later version.                                         --
--                                                                          --
--  This program is distributed in the hope that it will be useful, but     --
--  WITHOUT ANY WARRANTY; without even the implied warranty of              --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       --
--  General Public License for more details.                                --
--                                                                          --
--  You should have received a copy of the GNU General Public License       --
--  along with this library; if not, write to the Free Software Foundation, --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.          --
--                                                                          --
--  You are not allowed to use any part of this code to develop a program   --
--  whose output would be used to harass or prosecute other users of the    --
--  networks Adagio connects with. All data collected with Adagio or a tool --
--  containing Adagio code about other network users must remain            --
--  confidential and cannot be made public by any mean, nor be used to      --
--  harass or legally prosecute these users.                                --
------------------------------------------------------------------------------
--  $Id: adagio-ed2k.ads,v 1.3 2004/01/21 21:05:25 Jano Exp $

with Acf.Hash.Message_digests;
with Acf.Types;

package Adagio.Ed2k is

   Hash_block_size : constant := 9_728_000;

   subtype Hash_type is Acf.Hash.Message_digests.Message_digest;

   Null_hash : Acf.Hash.Message_digests.Message_digest 
      renames Acf.Hash.Message_digests.Null_message_digest;

   Null_hash_as_bytes : Acf.Types.Byte_array (1 .. 16) := (others => 0);

   Incorrect_hash_length : Exception;

   ------------------------------------------------------------------------
   -- Hash_as_hex                                                        --
   ------------------------------------------------------------------------
   -- Convert a ed2k hash to its Hex representation:
   function Hash_as_hex (Hash : in Hash_type) return String;

   ------------------------------------------------------------------------
   -- Hash_as_char_array                                                 --
   ------------------------------------------------------------------------
   -- Convert a ed2k hash to a String containing its bytes.
   function Hash_as_char_array (Hash : in Hash_type) return String;

   ------------------------------------------------------------------------
   -- Hash_from_char_array                                               --
   ------------------------------------------------------------------------
   -- Obtains a digest from a String containing bytes.
   function Hash_from_char_array (S : in String) return Hash_type;

end Adagio.Ed2k;
